<?php

namespace Drupal\media_pdf_thumbnail\Manager;

use Drupal\Core\Cache\CacheTagsInvalidatorInterface;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\File\FileSystem;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\file\FileInterface;
use Drupal\media\MediaInterface;
use Exception;

/**
 * Class MediaPdfThumbnailImageManager.
 *
 * @package Drupal\media_pdf_thumbnail\Manager
 */
class MediaPdfThumbnailImageManager {

  const VALID_MIME_TYPE = ['application/pdf'];

  const GENERIC_FILENAME = 'generic.png';

  /**
   * MediaPdfThumbnailManager.
   *
   * @var \Drupal\media_pdf_thumbnail\Manager\MediaPdfThumbnailImagickManager
   */
  protected $mediaPdfThumbnailImagickManager;

  /**
   * EntityTypeManager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityTypeManager;

  /**
   * FileSystem.
   *
   * @var \Drupal\Core\File\FileSystem
   */
  protected $fileSystem;

  /**
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * @var \Drupal\Core\Cache\Cache
   */
  protected $cache;

  /**
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * MediaPdfThumbnailImageManager constructor.
   *
   * @param \Drupal\media_pdf_thumbnail\Manager\MediaPdfThumbnailImagickManager $mediaPdfThumbnailImagickManager
   * @param \Drupal\Core\Entity\EntityTypeManager $entityTypeManager
   * @param \Drupal\Core\File\FileSystem $fileSystem
   * @param \Drupal\Core\Config\ConfigFactory $configFactory
   * @param \Drupal\Core\Database\Connection $connection
   * @param \Drupal\Core\Cache\CacheTagsInvalidatorInterface $cache
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerChannelFactory
   */
  public function __construct(MediaPdfThumbnailImagickManager $mediaPdfThumbnailImagickManager, EntityTypeManager $entityTypeManager, FileSystem $fileSystem, ConfigFactory $configFactory, Connection $connection, CacheTagsInvalidatorInterface $cache, LoggerChannelFactoryInterface $loggerChannelFactory) {
    $this->mediaPdfThumbnailImagickManager = $mediaPdfThumbnailImagickManager;
    $this->entityTypeManager = $entityTypeManager;
    $this->fileSystem = $fileSystem;
    $this->configFactory = $configFactory;
    $this->connection = $connection;
    $this->cache = $cache;
    $this->logger = $loggerChannelFactory->get('Media pdf thumbnail');
  }

  /**
   * @param \Drupal\Core\Entity\EntityInterface $entity
   * @param $fieldName
   * @param int $page
   *
   * @return array|\Drupal\Core\Entity\EntityInterface[]|false|void|null
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function createThumbnail(EntityInterface $entity, $fieldName, $page = 1) {

    if (empty($entity)) {
      return FALSE;
    }

    if ($entity->hasField($fieldName) && !empty($entity->get($fieldName)->getValue())) {
      $fileEntity = $this->getFileEntity($entity->get($fieldName)->getValue()[0]['target_id']);
      if ($fileEntity && in_array($fileEntity->getMimeType(), self::VALID_MIME_TYPE)) {
        $pdfImage = $this->getPdfImage($entity, $fileEntity, $page);
        if (empty($pdfImage)) {
          $pdfImage = $this->createPdfImageEntity($entity, $fileEntity, $page, $this->createThumbnailFileEntity($this->generatePdfImage($fileEntity, $page)));
        }
        return $pdfImage;
      }
    }
  }

  /**
   * @param $fid
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function getFileEntity($fid) {
    return $this->entityTypeManager->getStorage('file')->load($fid);
  }

  /**
   * @param \Drupal\Core\Entity\EntityInterface $entity
   * @param \Drupal\file\FileInterface $fileEntity
   * @param $page
   *
   * @return array|null
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function getPdfImage(EntityInterface $entity, FileInterface $fileEntity, $page) {

    $pdfImageEntity = $this->entityTypeManager->getStorage('pdf_image_entity')->loadByProperties([
      'referenced_entity_type' => $entity->getEntityTypeId(),
      'referenced_entity_id' => $entity->id(),
      'referenced_entity_revision_id' => $entity->getLoadedRevisionId(),
      'referenced_entity_lang' => $entity->language()->getId(),
      'pdf_file_id' => $fileEntity->id(),
      'pdf_file_page' => $page,
    ]);

    if (!empty($pdfImageEntity)) {
      $pdfImageEntity = reset($pdfImageEntity);
      return [
        'pdf_id' => $pdfImageEntity->get('pdf_file_id')->value,
        'pdf_uri' => $pdfImageEntity->get('pdf_file_uri')->value,
        'image_id' => $pdfImageEntity->get('image_file_id')->value,
        'image_uri' => $pdfImageEntity->get('image_file_uri')->value,
      ];
    }
    return NULL;
  }

  /**
   * @param \Drupal\Core\Entity\EntityInterface $entity
   * @param \Drupal\Core\Entity\EntityInterface $fileEntity
   * @param $page
   * @param $imageFileInfo
   *
   * @return array|null
   */
  protected function createPdfImageEntity(EntityInterface $entity, EntityInterface $fileEntity, $page, $imageFileInfo) {

    try {

      $newPdfImageFile = $this->entityTypeManager->getStorage('pdf_image_entity')->create([
        'referenced_entity_type' => $entity->getEntityTypeId(),
        'referenced_entity_bundle' => $entity->bundle(),
        'referenced_entity_id' => $entity->id(),
        'referenced_entity_revision_id' => $entity->getLoadedRevisionId(),
        'referenced_entity_lang' => $entity->language()->getId(),
        'pdf_file_id' => $fileEntity->id(),
        'pdf_file_uri' => $fileEntity->getFileUri(),
        'pdf_file_page' => $page,
        'image_file_id' => $imageFileInfo['fid'],
        'image_file_uri' => $imageFileInfo['uri'],
      ]);

      $newPdfImageFile->save();
      return [
        'pdf_id' => $fileEntity->id(),
        'pdf_uri' => $fileEntity->getFileUri(),
        'pdf_page' => $page,
        'image_id' => $imageFileInfo['fid'],
        'image_uri' => $imageFileInfo['uri'],
      ];
    }
    catch (Exception $e) {
      $this->logger->error($e->getMessage());
      return NULL;
    }
  }

  /**
   * @param $fileUri
   *
   * @return array|void|null
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  protected function createThumbnailFileEntity($fileUri) {

    if (empty($fileUri)) {
      return;
    }

    $infos = pathinfo($fileUri);
    $fileEntity = $this->entityTypeManager->getStorage('file')->create([
      'uri' => $fileUri,
      'status' => FILE_STATUS_PERMANENT,
    ]);
    $fileEntity->save();
    return $fileEntity ? [
      'fid' => $fileEntity->id(),
      'filename' => $infos['filename'],
      'uri' => $fileUri,
    ] : NULL;
  }

  /**
   * @param \Drupal\file\FileInterface $fileEntity
   * @param $page
   *
   * @return mixed|null
   */
  protected function generatePdfImage(FileInterface $fileEntity, $page = 1) {
    $fileInfos = $this->getFileInfos($fileEntity);
    if (!empty($fileInfos['source']) && !empty($fileInfos['destination'])) {
      return $this->mediaPdfThumbnailImagickManager->generateImageFromPDF($fileInfos['source'], $fileInfos['destination'], $page);
    }
    return NULL;
  }

  /**
   * @param \Drupal\media\MediaInterface $media
   * @param $fileEntityInfo
   * @param $thumbnailType
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   * @throws \Exception
   */
  /*  protected function setImageToMediaThumbnail(MediaInterface $media, $fileEntityInfo, $thumbnailType) {

      $lang = $media->language()->getId();
      $revisionId = $media->getLoadedRevisionId();

    /*  $revisionFieldValue = $media->getTranslation($lang)->get('vid')->get(0)->getValue();
      $vid = !empty($revisionFieldValue['value']) ? $revisionFieldValue['value'] : NULL;*/

  /* $mediaRevision = $this->entityTypeManager->getStorage('media')->loadRevision($vid);
   $mediaRevision = $mediaRevision->getTranslation($lang);

   $currentThumbnailFileEntity = $mediaRevision->get('thumbnail')->entity;
   $currentThumbnailFileUri = $currentThumbnailFileEntity ? $currentThumbnailFileEntity->getFileUri() : NULL;
   $thumbnailFileEntity = !empty($fileEntityInfo['fid']) ? $this->entityTypeManager->getStorage('file')->load($fileEntityInfo['fid']) : NULL;
   $thumbnailFileUri = $thumbnailFileEntity ? $thumbnailFileEntity->getFileUri() : NULL;

   if ($currentThumbnailFileUri != $thumbnailFileUri) {
     $mediaRevision->set('thumbnail', $fileEntityInfo['fid']);
     $mediaRevision->setRevisionTranslationAffected(TRUE);

//     $mediaRevision->save();
     $this->entityTypeManager->getStorage($media->getEntityTypeId())->resetCache([$media->id()]);
     $this->cache->invalidateTags($media->getCacheTagsToInvalidate());
   }

   return;

   if (empty($revisionId)) {
     $currentThumbnailFileEntity = $media->get('thumbnail')->entity;
     $currentThumbnailFileUri = $currentThumbnailFileEntity ? $currentThumbnailFileEntity->getFileUri() : NULL;
     $thumbnailFileEntity = !empty($fileEntityInfo['fid']) ? $this->entityTypeManager->getStorage('file')->load($fileEntityInfo['fid']) : NULL;
     $thumbnailFileUri = $thumbnailFileEntity ? $thumbnailFileEntity->getFileUri() : NULL;

     if ($currentThumbnailFileUri != $thumbnailFileUri) {
       $media->set('thumbnail', $fileEntityInfo['fid']);
       //$mediaRevision->setRevisionTranslationAffected(TRUE);
       $media->save();
     }
   }

   // Manage revision.
   if ($revisionId) {

     // Handle file usage.
     $values = [
       'fid' => $fileEntityInfo['fid'],
       'module' => $media->bundle(),
       'type' => $media->getEntityTypeId(),
       'id' => $media->id(),
     ];

     if (!$this->entryExists('file_usage', $values)) {
       $query = $this->connection->insert('file_usage');
       $values['count'] = 1;
       $query->fields($values);
       $query->execute();
     }

     // Handle media field revision.
     $query = $this->connection->update('media_field_revision');
     $values = [
       'langcode' => $lang,
       'thumbnail__target_id' => $fileEntityInfo['fid'],
       'thumbnail__alt' => $fileEntityInfo['filename'],
     ];
     $query->fields($values);
     $query->condition('mid', $media->id());
     $query->condition('vid', $revisionId);
     $query->condition('langcode', $lang);
     $query->execute();

   }

   $this->entityTypeManager->getStorage($media->getEntityTypeId())->resetCache([$media->id()]);
   $this->cache->invalidateTags($media->getCacheTagsToInvalidate());
 }*/

  /**
   * @param \Drupal\file\FileInterface $fileEntity
   *
   * @return array
   */
  protected function getFileInfos(FileInterface $fileEntity) {
    $sourcePath = $fileEntity->getFileUri();
    $destinationPath = $sourcePath . '.jpeg';
    return ['source' => $sourcePath, 'destination' => $destinationPath];
  }

  /**
   * @return int|string|null
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  /*  protected function getGenericThumbnail() {
      $uri = $this->configFactory->get('media.settings')->get('icon_base_uri');
      $files = $this->entityTypeManager->getStorage('file')->loadByProperties(['uri' => $uri . '/' . self::GENERIC_FILENAME]);
      return !empty($files) ? reset($files)->id() : NULL;
    }*/

  /**
   * @param $table
   * @param $values
   *
   * @return bool
   */
  /*  protected function entryExists($table, $values) {
      $query = $this->connection->select($table, 't');
      foreach ($values as $key => $value) {
        $query->condition('t.' . $key, $value);
      }
      $query->fields('t', ['fid']);
      return !empty($query->execute()->fetchAssoc());
    }*/

}
